pub mod config;
pub mod state;
pub mod model;

use std::net::{SocketAddr, IpAddr, Ipv4Addr};
use std::sync::Arc;

use axum::{routing::get, Router};
use clap::Parser;
use sqlx::postgres::PgPoolOptions;
use tower_http::trace::TraceLayer;
use tracing::debug;

use crate::config::Config;
use crate::state::AppState;

#[tokio::main]
async fn main() {
    dotenvy::dotenv().ok();

    tracing_subscriber::fmt()
        .with_max_level(tracing::Level::DEBUG)
        .init();

    debug!("Reading configuration");

    let config = Config::parse();

    let pool = PgPoolOptions::new()
        .max_connections(500)
        .connect(&config.database)
        .await
        .unwrap();

    let state = Arc::new(AppState { pool });

    let app = Router::new()
        .route("/", get(|| async { "Hello, World!" }))
        .layer(TraceLayer::new_for_http())
        .with_state(state);

    let socket = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(0, 0, 0, 0)), config.port);
    axum::Server::bind(&socket)
        .serve(app.into_make_service())
        .await
        .unwrap();
}
